'use strict';

/**
 * RPS App
 * Definición del módulo principal de la aplicación.
 */

window.rpsApp = angular.module('rps', ['ionic', 'rps.controllers', 'angularMoment', 'ngAnimate']); // eliminado 'config'

/**
 * Método de ejecución en la fase 'run' de la aplicación, donde se inicializan algunos de los componentes necesarios
 * como las librerías Parse o Cordova.
 */
rpsApp.run(['$ionicPlatform', '$rootScope', '$state', 'SessionService', 'amMoment',

    function($ionicPlatform, $rootScope, $state, SessionService, amMoment) {

        // Parse.com initialization
        Parse.initialize('', '');

        SessionService.setUser(Parse.User.current());

        $ionicPlatform.ready(function() {
            // Oculta la barra de accesorios del teclado
            if(window.cordova) {
                if (window.cordova.plugins.Keyboard) {
                    cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                }

                // Limita la orientación de la aplicación a vertical (o portrait).
                screen.lockOrientation('portrait');
            }

            if(window.StatusBar) {
                // org.apache.cordova.statusbar
                StatusBar.styleDefault();
            }
        });

        // UI Router control de auteticación. Si se intenta acceder a una vista que requiera autenticación sin estar logueado
        // se redirige al usuario a la vista de login.
        $rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
            if (toState.data.authenticate && !Parse.User.current()) {
                // Usuario no autenticado
                $state.transitionTo('login');
                event.preventDefault();
            }
        });

        amMoment.changeLocale('es');
    }
]);

/**
 * Método que se ejecuta en la fase 'config' de la aplicación.
 */
rpsApp.config(function($stateProvider, $urlRouterProvider) {
    $stateProvider

    .state('login', {
        url: '/login',

        templateUrl: 'templates/users/login.html',
        controller: 'LoginCtrl',

        data: {
            authenticate: false
        }
    })

    .state('register', {
        url: '/register',

        templateUrl: 'templates/users/register.html',
        controller: 'RegisterCtrl',
        data: {
            authenticate: false
        }
    })

    .state('app', {
        url: '/app',
        abstract: true,
        templateUrl: 'templates/main.html',
        controller: 'AppCtrl'
    })

    .state('app.entryDetail', {
        url: '/entries/detail/:entryId',
        views: {
            'appContent' :{
                templateUrl: 'templates/entries/entry-detail.html',
                controller: 'EntryDetailCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.addEntry', {
        url: '/entries/add/:entryId',
        views: {
            'appContent' :{
                templateUrl: 'templates/entries/add-entry.html',
                controller: 'AddEntryCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.addStep', {
        url: '/entries/add-step/:entryId',
        views: {
            'appContent' :{
                templateUrl: 'templates/entries/add-step.html',
                controller: 'AddStepCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.wall', {
        url: '/rps/wall',
        views: {
            'appContent' :{
                templateUrl: 'templates/rps/wall.html',
                controller: 'WallCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.profile', {
        abstract: true,
        url: '/users/profile/:userId',
        views: {
            'appContent': {
                templateUrl: 'templates/users/profile.html',
                controller: 'ProfileCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.profile.entries', {
        url: '/entries', // url relativa a app.profile
        views: {
            profileContent: {
                templateUrl: 'templates/users/profile-entries.html',
                controller: 'ProfileEntriesCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.profile.followers', {
        url: '/followers', // url relativa a app.profile
        views: {
            profileContent: {
                templateUrl: 'templates/users/profile-followers.html',
                controller: 'ProfileFollowersCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.profile.contacts', {
        url: '/contacts', // url relativa a app.profile
        views: {
            profileContent: {
                templateUrl: 'templates/users/profile-contacts.html',
                controller: 'ProfileContactsCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.search', {
        abstract: true,
        url: '/search',
        views: {
            'appContent': {
                templateUrl: 'templates/rps/search.html',
                controller: 'SearchCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.search.entries', {
        url: '/entries', // url relativa a app.search
        views: {
            searchContent: {
                templateUrl: 'templates/rps/search-entries.html',
                controller: 'SearchEntriesCtrl'
            }
        },
        data: {
            authenticate: true
        }
    })

    .state('app.search.users', {
        url: '/users', // url relativa a app.search
        views: {
            searchContent: {
                templateUrl: 'templates/rps/search-users.html',
                controller: 'SearchUsersCtrl'
            }
        },
        data: {
            authenticate: true
        }
    });

    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/app/rps/wall');
});
