'use strict';

/**
 * @ngdoc function
 * @name rpsControllers.controller:AddStepCtrl
 * @description
 * # AddStepCtrl
 * Controlador utilizado para la creación de un nuevo paso de una entrada.
 */
rpsControllers.controller('AddStepCtrl', [
    '$scope',
    'EntryService',
    'ModalService',
    '$ionicLoading',
    '$ionicPopup',
    '$state',
    '$stateParams',
    '$jrCrop',

    function($scope, EntryService, ModalService, $ionicLoading, $ionicPopup, $state, $stateParams, $jrCrop) {

        /**
         * @private
         * Método privado que inicializa el valor del campo descriotion del paso
         * @param step {Object} Contiene los datos del paso al que referencia
         */
        function init (step) {
            $scope.picData = undefined;
            if (step) {
                step.description = '';
            }
        }

        /**
         * @private
         * Método privado que inicializa el valor del campo descriotion del paso
         * @param step {Object} Contiene los datos del paso al que referencia
         */
        function save (step, back) {
            // Si se ha pulsado 'Nuevo paso' se comprueba que se ha agradado una imagen
            if (!$scope.picData || !step) {
                $scope.modalInfo = {
                    title: 'Ups!',
                    message: 'Necesitas añadir la imagen y la descripción al nuevo paso antes de proceder.'
                };
                ModalService.showAlert($scope);
            } else {
                $ionicLoading.show({templateUrl: 'templates/saving.html'});
                EntryService.addStep($stateParams.entryId, step.description, $scope.picData)
                    .then(function () {
                        $ionicLoading.hide();

                        if (back) {                          
                          $state.go('app.addEntry', {entryId: $stateParams.entryId});
                        } else {
                          // Inicializa los campos para añadir un nuevo paso sin necesidad de 
                          // hacer un $state.go()
                          init(step);
                          window.plugins.toast.showShortBottom('Paso guardado.');
                        }
                    })
                    .fail(function (error) {
                        $scope.modalInfo = {
                            title: 'Ups!',
                            message: error
                        };
                        ModalService.showAlert($scope);
                        $ionicLoading.hide();
                    });
            }
        }

        /**
         * Guarda el paso en el servidor
         * @param step {Object} Contiene los datos del paso al que referencia
         */
        $scope.save = function (step) {
            save (step, false);
        };

        /**
         * Vuelve a la vista de creación de entrada. Previamente, lanza un 'popup' para
         * que el usuario indice si quiere guardar previamente el paso o no. En caso
         * afirmativo, llama al servicio para que guarde el paso y vuelve a la vista de
         * creación de la entrada.
         * @param step {Object} Contiene los datos del paso al que referencia
         */
        $scope.back = function (step) {
            $ionicPopup.confirm({
                title: '¿Guardar paso?',
                template: '¿Quieres guardar el paso antes de volver?',
                scope: $scope,
                buttons: [
                  { text: 'Cancelar' },
                  { text: 'No',
                    onTap: function() {
                        $state.go('app.addEntry', {entryId: $stateParams.entryId});
                    }
                  },
                  {
                    text: 'Sí',
                    type: 'button-inverse',
                    onTap: function() {
                      save (step, true);
                    }
                  }
                ]
            });
        };

        /**
         * Hace una llamada al recurso que gestiona la cámara del dispositivo
         * @param type {Number} Indica si la imagen se tomará de la cámara o de la galería
         */
        $scope.takePic = function(type) {
            var options =   {
                saveToPhotoAlbum: true,
                quality: 50,
                destinationType: 1, // 0: DATA_URI, 1: FILE_URI, 2: NATIVE_URI
                sourceType: type,      // 0:Photo Library, 1=Camera, 2=Saved Photo Album
                encodingType: 0     // 0=JPG 1=PNG
            };
            navigator.camera.getPicture(onSuccess,onFail,options);
        };

        /**
         * @private
         * Método privado que se llama desde el método 'takePic' en caso de que se haya tomado
         * la imagen correctamente
         * @param file {Object} Contiene la imagen que se ha tomado del dispositivo
         */
        var onSuccess = function(file) {
            $jrCrop.crop({
                url: file,
                width: 300,
                height: 240,
                cancelText: 'Cancelar',
                chooseText: 'Recortar',
            }).then(function(canvas) {
                var quality = 0.4;
                $scope.picData = canvas.toDataURL('image/jpeg', quality);
            }, function() {
                // User canceled or couldn't load image.
            });


        };

        /**
         * @private
         * Método privado que se llama desde el método 'takePic' en caso de que se haya producido
         * un error al tomar la imagen del dispositivo
         * la imagen correctamente
         * @param e {Object} Contiene el error que se ha producido
         */
        var onFail = function(e) {
            console.log('camera onFail:', e);
        };
    }
]);
