'use strict';

/**
 * @ngdoc function
 * @name rpsControllers.controller:EntryDetailCtrl
 * @description
 * # EntryDetailCtrl
 * Controlador utilizado para la visualización de una entrada.
 */
rpsControllers.controller('EntryDetailCtrl', [
    '$scope',
    'EntryService',
    'ModalService',
    '$ionicLoading',
    '$ionicSlideBoxDelegate',
    '$state',
    '$stateParams',
    '$timeout',

    function($scope, EntryService, ModalService, $ionicLoading, $ionicSlideBoxDelegate, $state, $stateParams, $timeout) {
        $scope.showValidationMessages = false;

        $ionicLoading.show({
          templateUrl: 'templates/loading.html'
        });

        $scope.showSteps = false;
        $scope.comment = {content: ''};

        // LLamamos al método getEntryById del servicio para obtener la información de la entrada
        // que se quiere visualizar
        EntryService.getEntryById($stateParams.entryId)
            .then(function (entry) {
                $scope.entry = entry;
                $timeout(function () {
                    $scope.showSteps = true;
                }, 50);
                $ionicLoading.hide();
            })
            .fail(function (error) {
                $scope.modalInfo = {
                    title: 'Ups!',
                    message: error
                };
                ModalService.showAlert($scope);
                $ionicLoading.hide();
            });

        /**
         * Método necesario desde la vista para que se visualice correctamente el componente
         * SlideBox utilizado para realizar un scroll lateral de los pasos de la entrada
         */
        $scope.updateSlider = function () {
            $ionicSlideBoxDelegate.update();
        };

        /**
         * Añade un comentario a la entrada
         * @param isInvalid {Boolean} Indica si el formulario del comentario es válido o no
         */
        $scope.newComment = function (isInvalid) {
            //$state.go('app.entryComment');
            $scope.showValidationMessages = true;

            if (!isInvalid) {
                $ionicLoading.show({templateUrl: 'templates/saving.html'});
                EntryService.addComment($stateParams.entryId, $scope.comment.content)
                    .then(function (comment) {
                        $ionicLoading.hide();
                        $scope.entry.comments.splice(0, 0, comment);
                        $scope.entry.commentsCount++;
                        $scope.showValidationMessages = false;
                        $scope.comment.content = '';

                        window.plugins.toast.showShortBottom('Comentario añadido');
                    })
                    .fail(function (error) {
                        $scope.modalInfo = {
                            title: 'Ups!',
                            message: error
                        };
                        ModalService.showAlert($scope);
                        $ionicLoading.hide();
                    });
            }
        };

        /**
         * Hace un 'like' sobre una entrada.
         */
        $scope.like = function () {
            $scope.entry.userLike = !$scope.entry.userLike;
            EntryService.likeEntry($stateParams.entryId)
                .then(function (count) {
                    $scope.$apply(function () {
                        if ($scope.entry.userLike) {
                            $scope.entry.likesCount = count;
                            window.plugins.toast.showShortBottom('Me gusta enviado.');
                        } else {
                            $scope.entry.likesCount = count;
                            window.plugins.toast.showShortBottom('Me gusta descartado.');
                        }
                    });

                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Ups!',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $ionicLoading.hide();
                });
        };

        /**
         * Hace un 'done' sobre una entrada.
         */
        $scope.done = function () {
            $scope.entry.userDone = !$scope.entry.userDone;
            EntryService.doneEntry($stateParams.entryId)
                .then(function () {
                    $scope.$apply(function () {
                        if ($scope.entry.userDone) {
                            window.plugins.toast.showShortBottom('\'Hecho\' enviado.');
                            $scope.entry.donesCount++;
                        } else {
                            window.plugins.toast.showShortBottom('\'Hecho\' descartado.');
                            if ($scope.entry.donesCount > 0) {
                                $scope.entry.donesCount--;
                            }
                        }
                    });

                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Ups!',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $ionicLoading.hide();
                });
        };
    }
]);
