'use strict';

/**
 * @ngdoc function
 * @name rpsControllers.controller:SearchEntriesCtrl
 * @description
 * # SearchEntriesCtrl
 * Controlador utilizado para las búsquedas de entradas
 */
rpsControllers.controller('SearchEntriesCtrl', [
    '$scope',
    '$state',
    '$stateParams',
    'UserService',
    'EntryService',
    'ModalService',

    function($scope, $state, $stateParams, UserService, EntryService, ModalService) {
        console.log('search entries');

        $scope.search = {entries: '', users: ''};
        $scope.entries = [];
        $scope.page = 0;
        $scope.results = false;

        $scope.$parent.searchTab = 'entries';

        /**
         * Realiza la busqueda de entradas
         */
        $scope.searchEntries = function () {
            if ($scope.search.entries) {
                console.log('searching entries with: ', $scope.search.entries);
                $scope.loading = true;

                EntryService.searchEntries($scope.search.entries)
                    .then(function (results) {
                        if (!results.entries.length) {
                            $scope.results = false;
                            $scope.modalInfo = {
                                title: '',
                                message: 'No se han encontrado entradas.'
                            };

                            ModalService.showAlert($scope);
                        } else {
                            $scope.pages = results.pages;
                            $scope.page++;
                            $scope.results = true;
                        }

                        $scope.$apply(function () {
                            $scope.entries = results.entries;
                            $scope.loading = false;
                        });

                    })
                    .fail(function (error) {
                        $scope.modalInfo = {
                            title: 'Error',
                            message: error
                        };

                        ModalService.showAlert($scope);
                        $scope.$apply(function () {
                            $scope.loading = false;
                        });
                    });
            }
        };

        /**
         * Método llamado cuando se llega al final de la lista para cargar más contenido
         */
        $scope.loadMoreEntries = function() {
            EntryService.searchEntries($scope.search.entries, $scope.page)
                .then(function (data){
                    if (data.result === 'ok') {
                        data.entries.forEach(function (entry) {
                            $scope.entries.push(entry);
                        });
                        $scope.pages = data.pages;
                        $scope.page++;
                    }

                    $scope.$broadcast('scroll.infiniteScrollComplete');
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Error',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $scope.pages = -1; // para indicar que no se continue el scroll infinito
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                });
        };

        /**
         * Método usado para saber si hay más contenido que cargar al final de la lista.
         */
        $scope.morePages = function () {
            if ($scope.results) {
                if ($scope.pages === -1) {
                    return false;
                }

                if (!$scope.pages) {
                    return true;
                } else {
                    if ($scope.pages > $scope.page) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }

        };

        
        /**
         * Redirige al usuario al detalle de la entrada, o a su edición, en caso de que
         * sea un borrador.
         */
        $scope.showEntry = function (entry) {
            if (entry.publish) {
                $state.go('app.entryDetail', {entryId: entry.id});
            } else {
                $state.go('app.addEntry', {entryId: entry.id});
            }
        };
    }
]);
