'use strict';

/**
 * @ngdoc function
 * @name rpsControllers.controller:SearchUsersCtrl
 * @description
 * # SearchUsersCtrl
 * Controlador utilizado para las búsquedas de usuarios
 */
rpsControllers.controller('SearchUsersCtrl', [
    '$scope',
    '$state',
    '$stateParams',
    'UserService',
    'ModalService',

    function($scope, $state, $stateParams, UserService, ModalService) {
        $scope.search = {users: ''};
        $scope.users = [];
        $scope.page = 0;
        $scope.results = false;

        $scope.$parent.searchTab = 'users';

        /**
         * Realiza la busqueda de usuarios
         */
        $scope.searchUsers = function () {
            console.log($scope.search.users);
            if ($scope.search.users) {
                $scope.loading = true;

                UserService.searchUsers($scope.search.users)
                    .then(function (results) {
                        if (!results.users.length) {
                            $scope.results = false;
                            $scope.modalInfo = {
                                title: '',
                                message: 'No se han encontrado usuarios.'
                            };

                            ModalService.showAlert($scope);
                        } else {
                            $scope.pages = results.pages;
                            $scope.page++;
                            $scope.results = true;
                        }

                        $scope.$apply(function () {
                            $scope.users = results.users;
                            $scope.loading = false;
                        });

                    })
                    .fail(function (error) {
                        $scope.modalInfo = {
                            title: 'Error',
                            message: error
                        };

                        ModalService.showAlert($scope);
                        $scope.$apply(function () {
                            $scope.loading = false;
                        });
                    });

            }
        };

        /**
         * Método llamado cuando se llega al final de la lista para cargar más contenido
         */
        $scope.loadMoreUsers = function() {
            UserService.searchUsers($scope.search.users, $scope.page)
                .then(function (data){
                    if (data.result === 'ok') {
                        data.users.forEach(function (user) {
                            $scope.users.push(user);
                        });
                        $scope.pages = data.pages;
                        $scope.page++;
                    }

                    $scope.$broadcast('scroll.infiniteScrollComplete');
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Error',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $scope.pages = -1; // para indicar que no se continue el scroll infinito
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                });
        };

        /**
         * Método usado para saber si hay más contenido que cargar al final de la lista.
         */
        $scope.morePages = function () {
            if ($scope.results) {
                if ($scope.pages === -1) {
                    return false;
                }

                if (!$scope.pages) {
                    return true;
                } else {
                    if ($scope.pages > $scope.page) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }

        };


    }
]);
