'use strict';

/**
* @ngdoc function
* @name rpsApp.service:ErrorsService
* @description
* # ErrorsService
* Servicio para la gestión de errores de la aplicación.
*/
rpsApp.factory('ErrorsService', [

function() {

    var service = {},
        errorMessages = {
            0: 'Se ha producido un error desconocido, por favor intentalo más tarde.',
            1: 'Se ha producido un error en el servidor, intentalo más tarde.',
            100: 'Ha ocurrido un error de conexión con el servidor.',
            101: 'El nombre de usuario o contraseña que has introducido no son correctos o has perdido la sesión. Accede de nuevo.',
            124: 'Parece que el servidor no está respondiendo como debería.',
            141: 'Se ha producido un error en el servidor, por favor intentalo más tarde.',
            154: 'Error de limitación de Parse.com',
            200: 'Tienes que introducir usuario y contraseña para acceder.',
            201: 'Tienes que introducir usuario y contraseña para acceder.',
            202: 'Ya existe un usuario con el nombre de usuario que has elegido.',
            203: 'Parece que ya te has registrado anteriormente con este correo electrónico.'
        };

    /**
     * Devuelve un mensaje de error en base al código recibido.
     * @param error {Number} Código de error.
     * @return {String} Mensaje correspondiente al error.
     */
    service.getErrorMessage = function (error) {
        var errorCode;

        console.error(error.message);

        if (error.code === 141) {
            try {
                var e = JSON.parse(error.message);
                errorCode = e.code;
            } catch (err) {
                // esto significa que ha habido un error en el código fuente del CloudCode
                errorCode = 0;
            }
        } else {
            errorCode = error.code;
        }

        if (errorMessages[errorCode]) {
            return errorMessages[errorCode];
        } else {
            return 'Error desconocido: ' + errorCode;
        }

    };

    return service;
}
]);
