'use strict';

/**
 * @ngdoc function
 * @name rpsControllers.controller:WallCtrl
 * @description
 * # WallCtrl
 * Controlador utilizado para mostrar las entradas en el muro
 */
rpsControllers.controller('WallCtrl', [
    '$scope',
    'EntryService',
    '$ionicLoading',
    'ModalService',
    '$state',

    function($scope, EntryService, $ionicLoading, ModalService, $state) {

        $scope.entries = [];
        $scope.page = 0;

        /**
         * Método llamado en la acción pull-to-refresh para actualizar la lista
         * de entradas.
         */
        $scope.refreshWall = function () {
            updateList()
                .then(function () {
                    // Ocultar el spinner
                    $scope.$broadcast('scroll.refreshComplete');
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Error',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $scope.$broadcast('scroll.refreshComplete');
                });
        };

        /**
         * Método llamado cuando se llega al final de la lista para cargar más contenido
         */
        $scope.loadMoreEntries = function() {
            EntryService.list($scope.page)
                .then(function (data){
                    if (data.result === 'ok') {
                        data.entries.forEach(function (entry) {
                            $scope.entries.push(entry);
                        });
                        $scope.pages = data.pages;
                        $scope.page += 1;
                    }

                    $scope.$broadcast('scroll.infiniteScrollComplete');
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Error',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $scope.pages = -1; // para indicar que no se continue el scroll infinito
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                });
        };

        /**
         * Método usado para saber si hay más contenido que cargar al final de la lista.
         */
        $scope.morePages = function () {
            if ($scope.pages === -1) {
                return false;
            }

            if (!$scope.pages) {
                return true;
            } else {
                if ($scope.pages > $scope.page) {
                    return true;
                } else {
                    return false;
                }
            }

        };

        /**
         * @private
         * Actualiza la lista de entradas del muro.
         */
        function updateList() {
            return EntryService.list(0)
                .then(function (data){
                    if (data.result === 'ok') {
                        $scope.entries = data.entries;
                        $scope.pages = data.pages;
                        $scope.page = 1;
                    }
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Error',
                        message: error
                    };

                    ModalService.showAlert($scope);
                });
        }

        /**
         * Redirige al usuario al detalle de la entrada, o a su edición, en caso de que
         * sea un borrador.
         */
        $scope.showEntry = function (entry) {
            $state.go('app.entryDetail', {entryId: entry.id});
        };
    }
]);
