'use strict';

rpsControllers.controller('ProfileContactsCtrl', [
    '$scope',
    '$state',
    '$stateParams',
    'UserService',
    'ModalService',

    function($scope, $state, $stateParams, UserService, ModalService) {
        var promise;

        $scope.loading = true;
        $scope.$parent.profileTab = 'contacts';
        if ($stateParams.userId) {
            promise = UserService.getUserContacts($stateParams.userId);
        } else {
            promise = UserService.getMyContacts();
            $scope.myProfile = true;
        }

        // obtenemos la lista de contactos del usuario
        promise
            .then(function (contacts) {
                $scope.$apply(function () {
                    if (!contacts.length) {
                        $scope.nocontacts = true;
                    }
                    $scope.loading = false;
                    $scope.contacts = contacts;
                });
            })
            .fail(function (error) {
                $scope.modalInfo = {
                    title: 'Ups!',
                    message: error
                };
                ModalService.showAlert($scope);
            });

        /**
        * Hace 'unfollow' sobre un usuario.
        * @param contact {Object} Datos de contacto de la persona a dejar de seguir
        */
        $scope.unfollow = function (contact) {
            UserService.unfollowUser(contact.id)
                .then(function () {
                    $scope.$apply(function () {
                        var index = $scope.contacts.indexOf(contact);
                        $scope.contacts.splice(index, 1);
                    });

                    window.plugins.toast.showShortBottom('Ya no sigues a ' + $scope.user.username);
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Ups!',
                        message: error
                    };

                    ModalService.showAlert($scope);
                });
        };

        /**
         * Se muestra la vista de entradas del usuario
         * @param userId {Number} el identificador del usuario
         */
        $scope.viewProfile = function (userId) {
            $state.go('app.profile.entries', {userId: userId});
        };
    }
]);
