'use strict';

rpsControllers.controller('ProfileEntriesCtrl', [
    '$scope',
    '$state',
    '$stateParams',
    'UserService',
    'EntryService',
    'ModalService',

    function($scope, $state, $stateParams, UserService, EntryService, ModalService) {

        if ($stateParams.userId) {
            $scope.userId = $stateParams.userId;
        } else {
            $scope.userId = $scope.user.id;
            $scope.myProfile = true;
        }

        $scope.$parent.profileTab = 'entries';
        $scope.entries = [];
        $scope.page = 0;

        /**
         * Método llamado cuando se llega al final de la lista para cargar más contenido
         * También se llama al cargar la vista
         */
        $scope.loadMoreEntries = function() {
            EntryService.getUserEntries($scope.page, $scope.userId)
                .then(function (data){
                    if (data.result === 'ok') {
                        $scope.$apply(function () {
                            data.entries.forEach(function (entry) {
                                $scope.entries.push(entry);
                            });
                            $scope.pages = data.pages;
                            if (data.entries.length){
                                $scope.page += 1;
                            } else {
                                $scope.noentries = true;
                            }
                        });
                    }

                    $scope.$broadcast('scroll.infiniteScrollComplete');
                })
                .fail(function (error) {
                    $scope.modalInfo = {
                        title: 'Error',
                        message: error
                    };
                    ModalService.showAlert($scope);
                    $scope.pages = -1; // para indicar que no se continue el scroll infinito
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                });
        };


        /**
        * Método usado para saber si hay más contenido que cargar al final de la lista.
        */
        $scope.morePages = function () {
            if ($scope.pages <= 0) {
                return false;
            }

            if (!$scope.pages) {
                return true;
            } else {
                if ($scope.pages > $scope.page) {
                    return true;
                } else {
                    return false;
                }
            }

        };

        /**
         * Redirige al usuario al detalle de la entrada, o a su edición, en caso de que
         * sea un borrador.
         */
        $scope.showEntry = function (entry) {
            if (entry.publish) {
                $state.go('app.entryDetail', {entryId: entry.id});
            } else {
                $state.go('app.addEntry', {entryId: entry.id});
            }
        };

    }
]);
