/*global Parse*/
'use strict';

describe('AddEntryCtrl', function () {

    // load the controller's module
    beforeEach(module('rps'));

    var AddEntryCtrl,
        scope,
        state,
        entryService,
        modalService,
        ionicLoading,
        stateParams,
        ionicViewService,
        jrCrop,
        entry = {id: 11, title: 'Title', description: 'description', temporalTags: 'tags', imageThumb: 'thumb'};

    // Initialize the controller and a mock scope
    beforeEach(inject(function ($controller, $rootScope) {
        scope = $rootScope.$new();
        state = jasmine.createSpyObj('$state', ['go']);
        entryService = jasmine.createSpyObj('EntryService', ['getEntryById', 'deleteStep', 'addOrEdit', 'publish']);
        modalService = jasmine.createSpyObj('ModalService', ['showAlert']);
        ionicLoading = jasmine.createSpyObj('$ionicLoading', ['show', 'hide']);
        stateParams = {};
        ionicViewService = jasmine.createSpyObj('$ionicViewService', ['nextViewOptions']);
        jrCrop = jasmine.createSpyObj('$jrCrop', ['crop']);

        window.plugins = {};
        window.plugins.toast = jasmine.createSpyObj('toast', ['showShortBottom']);

        navigator.camera = jasmine.createSpyObj('camera', ['getPicture']);

        entryService.getEntryById.and.returnValue(Parse.Promise.as({}));
        entryService.addOrEdit.and.returnValue(Parse.Promise.as(entry));
        entryService.publish.and.returnValue(Parse.Promise.as());
        entryService.deleteStep.and.returnValue(Parse.Promise.as());
    }));

    describe('when stateParams has an entryId', function () {

        beforeEach(inject(function ($controller) {
            stateParams.entryId = '11';

            AddEntryCtrl = $controller('AddEntryCtrl', {
                $scope: scope,
                EntryService: entryService,
                ModalService: modalService,
                $ionicLoading: ionicLoading,
                $state: state,
                $stateParams: stateParams,
                $ionicViewService: ionicViewService,
                $jrCrop: jrCrop
            });
        }));

        it('should call getEntryById if it receives an entryId param', function () {
            expect(entryService.getEntryById).toHaveBeenCalledWith(stateParams.entryId);
        });

        it('should save an entry using EntryService.addOrEdit if it does not receive newStep and publish params', function () {

            scope.save(entry);

            expect(entryService.addOrEdit).toHaveBeenCalled();

            expect(window.plugins.toast.showShortBottom).toHaveBeenCalled();

        });

        it('should save and publish an entry if the entry has steps', function () {
            entry.steps = [{}];
            scope.publish(entry);

            expect(entryService.addOrEdit).toHaveBeenCalled();
            expect(entryService.publish).toHaveBeenCalledWith(entry);
            expect(state.go).toHaveBeenCalledWith('app.wall');
            expect(window.plugins.toast.showShortBottom).toHaveBeenCalled();

            entry.steps = undefined;

        });

        it('should not publish an entry if the entry does not have steps', function () {
            scope.publish(entry);

            expect(entryService.addOrEdit).not.toHaveBeenCalled();
        });

        it('should save an entry using EntryService.addOrEdit if it receives newStep and does not receive publish params', function () {

            scope.newStep(entry);

            expect(entryService.addOrEdit).toHaveBeenCalled();
            expect(state.go).toHaveBeenCalledWith('app.addStep', {entryId: entry.id});
            expect(window.plugins.toast.showShortBottom).toHaveBeenCalled();

        });

        it('should remove a step from the entry', function () {
            entry.steps = [{id: 1}];
            scope.entryData = entry;

            scope.removeStep(entry.steps[0]);

            expect(entryService.deleteStep).toHaveBeenCalledWith(1);
            expect(entry.steps.length).toBe(0);
            expect(window.plugins.toast.showShortBottom).toHaveBeenCalled();
        });

        it('should show a modal if there is no step', function () {
            scope.removeStep();

            expect(entryService.deleteStep).not.toHaveBeenCalled();
            expect(modalService.showAlert).toHaveBeenCalled();
        });

    });

    describe('when stateParams does not have an entryId', function () {

        beforeEach(inject(function ($controller) {
            AddEntryCtrl = $controller('AddEntryCtrl', {
                $scope: scope,
                EntryService: entryService,
                ModalService: modalService,
                $ionicLoading: ionicLoading,
                $state: state,
                $stateParams: stateParams,
                $ionicViewService: ionicViewService,
                $jrCrop: jrCrop
            });
        }));

        // No es necesario probar todos los métodos por que no es tarea del controlador
        // comprobar si hay un entry.id o no para crear o editar la entrada.

        it('should not call getEntryById if it does not receive entryId param', function () {
            expect(entryService.getEntryById).not.toHaveBeenCalled();
        });

        it('should call getPicture method to take a photo', function () {
            scope.takePic();

            expect(navigator.camera.getPicture).toHaveBeenCalled();
        });


    });

});
