/*global Parse, xit*/
'use strict';

describe('SearchEntriesCtrl', function () {

    // load the controller's module
    beforeEach(module('rps'));

    var SearchEntriesCtrl,
        scope,
        state,
        stateParams,
        entryService,
        modalService,
        results;

    // Initialize the controller and a mock scope
    beforeEach(inject(function ($controller, $rootScope) {
        scope = $rootScope.$new();
        spyOn(scope, '$broadcast');
        entryService = jasmine.createSpyObj('EntryService', ['searchEntries', 'loadMoreEntries', 'morePages', 'showEntry']);
        modalService = jasmine.createSpyObj('ModalService', ['showAlert']);
        state = jasmine.createSpyObj('$state', ['go']);
        stateParams = {};
        results = {
                result: 'ok',
                entries: []
            };

        entryService.searchEntries.and.returnValue(Parse.Promise.as(results));

        SearchEntriesCtrl = $controller('SearchEntriesCtrl', {
            $scope: scope,
            EntryService: entryService,
            ModalService: modalService,
            $state: state,
            $stateParams: stateParams
        });
    }));

    // TODO: $scope.$apply está haciendo que se hagan get de templates que no se necesitan aquí y
    // hace que este test falle. Hay que revisar el porqué, mientras tanto se omite este test.
    xit('should search entries if results is not empty', function () {


        results = {entries: ['entries']};
        scope.search = {entries: ['nombre']};

        scope.searchEntries();

        expect(entryService.searchEntries).toHaveBeenCalled();
        expect(scope.results).toBeTruthy();
    });

    // TODO: $scope.$apply está haciendo que se hagan get de templates que no se necesitan aquí y
    // hace que este test falle. Hay que revisar el porqué, mientras tanto se omite este test.
    xit('should not search entries if results is empty', function () {

        scope.search = {entries: ['nombre']};

        scope.searchEntries();

        expect(entryService.searchEntries).toHaveBeenCalled();
        expect(scope.results).toBeFalsy();
        expect(modalService.showAlert).toHaveBeenCalled();
    });

    describe('loadMoreEntries', function () {
        it('should load more entries ', function () {
            scope.page = 2;
            scope.loadMoreEntries();

            expect(scope.$broadcast).toHaveBeenCalledWith('scroll.infiniteScrollComplete');
            expect(entryService.searchEntries).toHaveBeenCalled();
            expect(scope.page).toBe(3);
        });
    });

    describe('morePages', function () {
        it('should return true if it exists more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = 3;       

            expect(scope.morePages()).toBeTruthy();
        });

        it('should return false if it does not exist more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = -1;

            expect(scope.morePages()).toBeFalsy();
        });

        it('should return false if it does not exist more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = undefined;

            expect(scope.morePages()).toBeTruthy();
        });
    });

    describe('showEntry', function () {
        it('should go to entryDetail state', function() {
            scope.showEntry({id: 1, publish: true});
            expect(state.go).toHaveBeenCalledWith('app.entryDetail', {entryId: 1});
        });

        it('should go to addEntry state', function() {
            scope.showEntry({id: 1, publish: false});
            expect(state.go).toHaveBeenCalledWith('app.addEntry', {entryId: 1});
        });
    });
});
