/*global Parse, xit*/
'use strict';

describe('SearchUsersCtrl', function () {

    // load the controller's module
    beforeEach(module('rps'));

    var SearchUsersCtrl,
        scope,
        state,
        stateParams,
        userService,
        modalService,
        results;

    // Initialize the controller and a mock scope
    beforeEach(inject(function ($controller, $rootScope) {
        scope = $rootScope.$new();
        spyOn(scope, '$broadcast');
        userService = jasmine.createSpyObj('UserService', ['searchUsers', 'loadMoreUsers', 'morePages', 'showEntry']);
        modalService = jasmine.createSpyObj('ModalService', ['showAlert']);
        state = jasmine.createSpyObj('$state', ['go']);
        stateParams = {};
        results = {
                result: 'ok',
                users: []
            };

        userService.searchUsers.and.returnValue(Parse.Promise.as(results));

        SearchUsersCtrl = $controller('SearchUsersCtrl', {
            $scope: scope,
            UserService: userService,
            ModalService: modalService,
            $state: state,
            $stateParams: stateParams
        });
    }));

    // TODO: $scope.$apply está haciendo que se hagan get de templates que no se necesitan aquí y
    // hace que este test falle. Hay que revisar el porqué, mientras tanto se omite este test.
    xit('should search users if results is not empty', function () {


        results = {users: ['users']};
        scope.search = {users: ['nombre']};

        scope.searchUsers();

        expect(userService.searchUsers).toHaveBeenCalled();
        expect(scope.results).toBeTruthy();
    });

    // TODO: $scope.$apply está haciendo que se hagan get de templates que no se necesitan aquí y
    // hace que este test falle. Hay que revisar el porqué, mientras tanto se omite este test.
    xit('should not search users if results is empty', function () {

        scope.search = {users: ['nombre']};

        scope.searchUsers();

        expect(userService.searchUsers).toHaveBeenCalled();
        expect(scope.results).toBeFalsy();
        expect(modalService.showAlert).toHaveBeenCalled();
    });

    describe('loadMoreUsers', function () {
        it('should load more users ', function () {
            scope.page = 2;
            scope.loadMoreUsers();

            expect(scope.$broadcast).toHaveBeenCalledWith('scroll.infiniteScrollComplete');
            expect(userService.searchUsers).toHaveBeenCalled();
            expect(scope.page).toBe(3);
        });
    });

    describe('morePages', function () {
        it('should return true if it exists more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = 3;       

            expect(scope.morePages()).toBeTruthy();
        });

        it('should return false if it does not exist more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = -1;

            expect(scope.morePages()).toBeFalsy();
        });

        it('should return false if it does not exist more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = undefined;

            expect(scope.morePages()).toBeTruthy();
        });
    });
});
