/*global Parse */
'use strict';

describe('WallCtrl', function () {

    // load the controller's module
    beforeEach(module('rps'));

    var EntryDetailCtrl,
        scope,
        entryService,
        modalService,
        ionicLoading,
        state,
        listEntries;

    // Initialize the controller and a mock scope
    beforeEach(inject(function ($controller, $rootScope) {
        listEntries = {result: 'ok', entries: []};
        scope = $rootScope.$new();
        state = jasmine.createSpyObj('$state', ['go']);
        entryService = jasmine.createSpyObj('EntryService', ['list']);
        modalService = jasmine.createSpyObj('ModalService', ['showAlert']);
        ionicLoading = jasmine.createSpyObj('$ionicLoading', ['show', 'hide']);
        spyOn(scope, '$broadcast');

        scope.page = 0;

        entryService.list.and.returnValue(Parse.Promise.as(listEntries));

        EntryDetailCtrl = $controller('WallCtrl', {
            $scope: scope,
            EntryService: entryService,
            ModalService: modalService,
            $ionicLoading: ionicLoading,
            $state: state
        });
    }));

    describe('refreshWall', function () {
        it('should broadcast a scroll.refreshComplete event', function () {
            scope.refreshWall();
            expect(scope.$broadcast).toHaveBeenCalledWith('scroll.refreshComplete');
        });
    });

    describe('loadMoreEntries', function () {
        it('should call EntryService.list method', function () {
            scope.loadMoreEntries();

            expect(entryService.list).toHaveBeenCalledWith(0);
            expect(scope.page).toBe(1);
            expect(scope.$broadcast).toHaveBeenCalledWith('scroll.infiniteScrollComplete');
        });
    });

    describe('showEntry', function () {
        it('should go to another state', function () {
            scope.showEntry({id: 1});

            expect(state.go).toHaveBeenCalledWith('app.entryDetail', {entryId: 1});
        });
    });

    describe('morePages', function () {
        it('should return true if it exists more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = 3;

            expect(scope.morePages()).toBeTruthy();
        });

        it('should return false if it does not exist more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = -1;

            expect(scope.morePages()).toBeFalsy();
        });

        it('should return false if it does not exist more pages', function () {
            scope.results = true;
            scope.page = 2;
            scope.pages = undefined;

            expect(scope.morePages()).toBeTruthy();
        });
    });

});
